
##' NanoStringQCPro example dataset
##'
##' Example data for the NanoStringQCPro package
##'
##' @name       example_rccSet
##' @docType    data
##' @format     An RccSet object
##' @source     This is an artificial dataset designed to resemble real data.
##' @author     Dorothee Nickles
##' @keywords   datasets
NULL

