% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NanoStringQualityMetrics.R
\name{myCols}
\alias{myCols}
\title{myCols}
\usage{
myCols()
}
\value{
A vector of colors
}
\description{
Function that defines nice colors
}
\author{
Dorothee Nickles
}
